(* Delphi dialog
   Edit password with confirmation

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - June 2005
   Vers. 2 - July 2022: define compiler switch "ACCESSIBLE" to make dialog
                        messages accessible to screenreaders
   last modified: July 2022
   *)
   
unit NewPwdDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls;

const
  defMinPwdLength = 8;

type
  TNewPwdDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    btnShow: TSpeedButton;
    paPwd: TPanel;
    paBottom: TPanel;
    paConfirm: TPanel;
    edtPwd: TLabeledEdit;
    edtRepPwd: TLabeledEdit;
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure btnShowClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    { Private declarations }
    FConf   : boolean;
    hgt,
    FMinLen : integer;
  public
    { Public declarations }
{$IFDEF HDPI}   // scale glyphs and images for High DPI
    procedure AfterConstruction; override;
{$EndIf}
    function Execute (const Title : string; var Pwd : String;
                      Confirm : boolean; Show : boolean = false;
                      MinLen : integer = defMinPwdLength) : boolean; overload;
    function Execute (const Title : string; var Pwd : AnsiString;
                      Confirm : boolean; Show : boolean = false;
                      MinLen : integer = defMinPwdLength) : boolean; overload;
    function Execute (APos : TPoint; const Title : string; var Pwd : String;
                      Confirm : boolean; Show : boolean = false;
                      MinLen : integer = defMinPwdLength) : boolean; overload;
    function Execute (APos : TPoint; const Title : string; var Pwd : AnsiString;
                      Confirm : boolean; Show : boolean = false;
                      MinLen : integer = defMinPwdLength) : boolean; overload;
  end;

// Ansi password (no unicode)
function InputNewPassword (APos : TPoint; const Title : string; var Pwd : AnsiString;
                           Confirm : boolean; Show : boolean = false;
                           MinLen : integer = defMinPwdLength) : boolean; overload;

function InputNewPassword (const Title : string; var Pwd : AnsiString;
                           Confirm : boolean; Show : boolean = false;
                           MinLen : integer = defMinPwdLength) : boolean; overload;

// Unicode password
function InputNewPassword (APos : TPoint; const Title : string; var Pwd : String;
                           Confirm : boolean; Show : boolean = false;
                           MinLen : integer = defMinPwdLength) : boolean; overload;

function InputNewPassword (const Title : string; var Pwd : String;
                           Confirm : boolean; Show : boolean = false;
                           MinLen : integer = defMinPwdLength) : boolean; overload;

function GetPassword (APos : TPoint; const Title : string; var Pwd : string) : boolean; overload;
function GetPassword (APos : TPoint; const Title : string; var Pwd : AnsiString) : boolean; overload;

function GetPassword (const Title : string; var Pwd : String) : boolean; overload;
function GetPassword (const Title : string; var Pwd : AnsiString) : boolean; overload;

var
  NewPwdDialog: TNewPwdDialog;

implementation

{$R *.DFM}

uses GnuGetText, ExtSysUtils, WinUtils, {$IFDEF ACCESSIBLE} ShowMessageDlg {$ELSE} MsgDialogs {$ENDIF};

{ ------------------------------------------------------------------- }
procedure TNewPwdDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  hgt:=paConfirm.Height;
  end;

procedure TNewPwdDialog.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
{$IFDEF ACCESSIBLE}
  if (Key=VK_F11) then begin
    if ActiveControl is TCustomEdit then begin
      with (ActiveControl as TCustomEdit) do if length(TextHint)>0 then ShowHintInfo(TextHint)
      else if length(Hint)>0 then ShowHintInfo(Hint);
      end
    else with ActiveControl do if length(Hint)>0 then ShowHintInfo(Hint);
    end;
{$ENDIF}
  end;

{$IFDEF HDPI}   // scale glyphs and images for High DPI
procedure TNewPwdDialog.AfterConstruction;
begin
  inherited;
  if Application.Tag=0 then
    ScaleButtonGlyphs(self,PixelsPerInchOnDesign,Monitor.PixelsPerInch);
  end;
{$EndIf}

procedure TNewPwdDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if (ModalResult=mrOK) and FConf then
    CanClose:=(edtPwd.Text=edtRepPwd.Text)
               and ((length(edtPwd.Text)>=FMinLen) or (length(edtPwd.Text)=0));
  if not CanClose then begin
    if length(edtPwd.Text)<FMinLen then begin
      ErrorDialog(TryFormat(dgettext('dialogs','Minimum length of password is %u characters'),[FMinLen]));
      edtPwd.SetFocus;
      end
    else begin
      ErrorDialog(dgettext('dialogs','Passwords do not match!'));
      edtRepPwd.SetFocus;
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TNewPwdDialog.btnShowClick(Sender: TObject);
var
  c : char;
begin
  if btnShow.Down then c:=#0 else c:='*';
  edtPwd.PasswordChar:=c;
  edtRepPwd.PasswordChar:=c;
  end;

{ ------------------------------------------------------------------- }
(* Passworteingabe, Ergebnis: "true" bei "ok" *)
function TNewPwdDialog.Execute (APos : TPoint; const Title : string; var Pwd : String;
                                Confirm,Show : boolean; MinLen : integer) : boolean;
var
  h : integer;
begin
  AdjustFormPosition(Screen,self,APos);
  Caption:=Title;
  if Show then begin
    edtPwd.Text:=Pwd;
    edtRepPwd.Text:=Pwd;
    end
  else begin
    edtPwd.Text:='';
    edtRepPwd.Text:='';
    end;
  FConf:=Confirm;
  FMinLen:=MinLen;
  btnShow.Visible:=Show;
  h:=paPwd.Height+paBottom.Height;
//  paConfirm.Visible:=FConf;
  if FConf then ClientHeight:=h+hgt else ClientHeight:=h;
  edtRepPwd.TabStop:=FConf;
  ActiveControl:=edtPwd;
  if ShowModal=mrOK then begin
    Pwd:=edtPwd.Text;
    Result:=true;
    end
  else Result:=false;
  end;

function TNewPwdDialog.Execute (APos : TPoint; const Title : string; var Pwd : AnsiString;
                      Confirm,Show : boolean; MinLen : integer) : boolean;
var
  s : string;
begin
  Result:=Execute (APos,Title,s,Confirm,Show,MinLen);
  if Result then Pwd:=s;
  end;

function TNewPwdDialog.Execute (const Title  : string; var Pwd : String;
                                Confirm,Show : boolean; MinLen : integer) : boolean;
begin
  Result:=Execute(CenterPos,Title,Pwd,Confirm,Show,MinLen);
  end;

function TNewPwdDialog.Execute (const Title  : string; var Pwd : AnsiString;
                                Confirm,Show : boolean; MinLen : integer) : boolean;
begin
  Result:=Execute(CenterPos,Title,Pwd,Confirm,Show,MinLen);
  end;

{ ------------------------------------------------------------------- }
function InputNewPassword (APos : TPoint; const Title : string; var Pwd : AnsiString;
                           Confirm,Show : boolean; MinLen : integer) : boolean;
begin
  if not assigned(NewPwdDialog) then NewPwdDialog:=TNewPwdDialog.Create(Application);
  Result:=NewPwdDialog.Execute(APos,Title,Pwd,Confirm,Show,MinLen);
  FreeAndNil(NewPwdDialog);
  end;

function InputNewPassword (const Title : string; var Pwd : AnsiString;
                           Confirm,Show : boolean; MinLen : integer) : boolean;
begin
  Result:=InputNewPassword(CenterPos,Title,Pwd,Confirm,Show,MinLen);
  end;

function InputNewPassword (APos : TPoint; const Title : string; var Pwd : String;
                           Confirm,Show : boolean; MinLen : integer) : boolean;
begin
  if not assigned(NewPwdDialog) then NewPwdDialog:=TNewPwdDialog.Create(Application);
  Result:=NewPwdDialog.Execute(APos,Title,Pwd,Confirm,Show,MinLen);
  FreeAndNil(NewPwdDialog);
  end;

function InputNewPassword (const Title : string; var Pwd : String;
                           Confirm,Show : boolean; MinLen : integer) : boolean;
begin
  Result:=InputNewPassword(CenterPos,Title,Pwd,Confirm,Show,MinLen);
  end;

function GetPassword (APos : TPoint; const Title : string; var Pwd : String) : boolean;
begin
  if not assigned(NewPwdDialog) then NewPwdDialog:=TNewPwdDialog.create(Application);
  Result:=NewPwdDialog.Execute(APos,Title,Pwd,false);
  FreeAndNil(NewPwdDialog);
  end;

function GetPassword (APos : TPoint; const Title : string; var Pwd : AnsiString) : boolean;
begin
  if not assigned(NewPwdDialog) then NewPwdDialog:=TNewPwdDialog.create(Application);
  Result:=NewPwdDialog.Execute(APos,Title,Pwd,false);
  FreeAndNil(NewPwdDialog);
  end;

function GetPassword (const Title : string; var Pwd : String) : boolean;
begin
  if not assigned(NewPwdDialog) then NewPwdDialog:=TNewPwdDialog.create(Application);
  Result:=NewPwdDialog.Execute(Title,Pwd,false);
  FreeAndNil(NewPwdDialog);
  end;

function GetPassword (const Title : string; var Pwd : AnsiString) : boolean;
begin
  if not assigned(NewPwdDialog) then NewPwdDialog:=TNewPwdDialog.create(Application);
  Result:=NewPwdDialog.Execute(Title,Pwd,false);
  FreeAndNil(NewPwdDialog);
  end;

end.
