(* Delphi Dialog
   Web Updater
   ===========
   - Check Version on Web for Update (Thread based)
   - Download file from URL and show Progress

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1   - Jul. 2008
         1.1 - Sep. 2016: Error handling
   Vers. 2   - Aug. 2018: optional verification with MD5 or SHA256
                          CheckHash = 0 : no verification
                                    = 1 : MD5
                                    = 2 : SHA256
   Vers. 3   - Jan. 2020: SSL support added
   last modified:  August 2022
   *)

unit WebUpdateDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Vcl.ComCtrls,
  IdComponent, IdHttp, Prgrss;

const
  defHttpProxyPort = 80;
  LocExt = 'loc';
  Md5Ext = 'md5';
  ShaExt = 'sha256';

type
  THttpProxyParams = record
    UseProxy       : boolean;
    Server,
    Username,
    Password       : string;
    Port           : integer;
    end;

  THttpSSLParams = record
    Path,Root,Cert,Key : string;
    CertPwd            : AnsiString;
    end;

  TUpdateStatus = (usNotAvail,usAvail,usCancel,usNoServer);
  TStatusEvent = procedure (var IsIdle : boolean) of object;

  TExtHttp = class (TIdHttp)
  private
    fRoot,fCert,fKey : string;
    fCertPwd         : AnsiString;
    fUseSSL          : boolean;
    procedure InitIOHandler;
  public
    constructor Create (AOwner : TComponent; const Path,Root,Cert,Key : string; Pwd : AnsiString); overload;
    constructor Create (AOwner : TComponent); overload;
    end;

  TUpdateThread = class (TThread)
  private
    FUrl,FVerName,
    FVersion,
    FErrorMsg      : string;
    FLevel         : integer;
    IdHTTP         : TExtHttp;
    FCanTerminate,
    FCancelled,
    FShowFault     : boolean;
    FUpdateStatus  : TUpdateStatus;
    FOnTermCheck   : TStatusEvent;
    procedure SetTermCheck (ACallBack : TStatusEvent);
    procedure DoTermCheck;
  protected
    procedure Execute; override;
  public
    constructor Create (const AUrl,AVerName,Version : string; const AProxyPar : THttpProxyParams;
                        const ASSLParams : THttpSSLParams; ALevel : integer;
                        AOnTermCheck : TStatusEvent; AOnTerminate :TNotifyEvent; AShowFault : boolean);
    destructor Destroy; override;
    function GetDownloadInfo (var AUrl,AName : string; const AltUrl : string ='') : boolean;
    procedure Cancel;
    property ErrorMessage : string read FErrorMsg;
    property UpdateVersion : string read FVersion;
    property ShowFault : boolean read FShowFault;
    property UpdateStatus : TUpdateStatus read FUpdateStatus;
    property OnCheckTerminated : TStatusEvent read FOnTermCheck write SetTermCheck;
    property Terminated;
    end;

  TDownload = class(TForm)
    edURL: TLabeledEdit;
    edFilename: TLabeledEdit;
    pbDownload: TNewProgressBar;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
  private
    { Private declarations }
    nMax      : int64;
    IdHTTP    : TExtHttp;
    procedure StartWork(Sender: TObject; AWorkMode: TWorkMode; AWorkCount: Int64);
    procedure ShowWork(Sender: TObject; AWorkMode: TWorkMode; AWorkCount: Int64);
    procedure FinishWork(Sender: TObject; AWorkMode: TWorkMode);
  public
    { Public declarations }
    function Execute (const ATitle,AURL,AFilename : string;
                      const AProxyPar : THttpProxyParams;
                      const ASSLParams : THttpSSLParams;
                      CheckHash : integer; var ErrorMsg : string) : boolean;
  end;

function IsNewerVersion (v1,v2 : string; Level : integer) : boolean;
function DefProxy : THttpProxyParams;

function SSLParams (const APath,ARoot,ACert,AKey : string; APwd : AnsiString) : THttpSSLParams;
function NoSSLParams : THttpSSLParams;

function PerformDownload (const ATitle,AURL,AFilename : string;
                          const AProxyPar : THttpProxyParams;
                          const ASSLParams : THttpSSLParams;
                          CheckHash : integer; var ErrorMsg : string) : boolean;

var
  Download: TDownload;

implementation

{$R *.dfm}

uses GnuGetText, System.StrUtils, StringUtils, FileUtils, PathUtils, Hashes, IdException,
  ExtSysUtils, IdResourceStringsProtocols, IdResourceStringsSSPI, IdSSLOpenSSL, IdIOHandler
  {$IFDEF ACCESSIBLE}, ShowMessageDlg {$ENDIF};

{ ---------------------------------------------------------------- }
constructor TExtHttp.Create (AOwner : TComponent; const Path,Root,Cert,Key : string; Pwd : AnsiString);
begin
  inherited Create(AOwner);
  fUseSSL:=(length(Cert)>0) and (length(Key)>0) and (length(Root)>0);
  if fUseSSL then begin
    try
      TIdSSLContext.Create.Free;
      fUseSSL:=true;
    except
      fUseSSL:=false;
      end;
    if fUseSSL then begin
      fRoot:=Path+Root; fCert:=Path+Cert; fKey:=Path+Key;
      fUseSSL:=FileExists(fRoot) and FileExists(fCert) and FileExists(fKey);
      end;
    end;
  fCertPwd:=Pwd;
  InitIOHandler;
  end;

constructor TExtHttp.Create (AOwner : TComponent);
begin
  Create(AOwner,'','','','','');
  end;

procedure TExtHttp.InitIOHandler;
begin
  if IOHandler<>nil then Exit;
  if fUseSSL then begin
    IOHandler:=TIdSSLIOHandlerSocketOpenSSL.Create(self);
    with (IOHandler as TIdSSLIOHandlerSocketOpenSSL) do begin
      with SSLOptions do begin
        RootCertFile:=fRoot; CertFile:=fCert; KeyFile:=fKey;
        SSLVersions:=[sslvTLSv1,sslvTLSv1_1,sslvTLSv1_2];
//        VerifyMode:=[sslvrfPeer];
        end;
      OnGetPassword:=nil;
//      OnVerifyPeer:=VerifyPeer;
      end;
    end
  else IOHandler:=TIdIOHandler.MakeDefaultIOHandler(self);
  ManagedIOHandler:=True;
  end;

{ ---------------------------------------------------------------- }
// True: v1<v2
function IsNewerVersion (v1,v2 : string; Level : integer) : boolean;
var
  n1,n2,i : integer;
begin
  n1:=0; i:=0;                               
  while (length(v1)>0) and (i<level) do begin
    n1:=100*n1+ReadNxtInt(v1,'.',0);
    inc(i);
    end;
  while (i<level) do begin
    n1:=100*n1;
    inc(i);
    end;
  n2:=0; i:=0;
  while (length(v2)>0) and (i<level) do begin
    n2:=100*n2+ReadNxtInt(v2,'.',0);
    inc(i);
    end;
  while (i<level) do begin
    n1:=100*n1;
    inc(i);
    end;
  Result:=n1<n2;
  end;

// Reset proxy settings
function DefProxy : THttpProxyParams;
begin
  with Result do begin
    UseProxy:=false;
    Server:='';
    Username:='';
    Password:='';
    Port:=defHttpProxyPort;
    end;
  end;

// set parameter for SSL
function SSLParams (const APath,ARoot,ACert,AKey : string; APwd : AnsiString) : THttpSSLParams;
begin
  with Result do begin
    Path:=APath; Root:=ARooT; Cert:=ACert; Key:=AKey; CertPwd:=APwd;
    end;
  end;

function NoSSLParams : THttpSSLParams;
begin
  with Result do begin
    Path:=''; Root:=''; Cert:=''; Key:=''; CertPwd:='';
    end;
  end;

{ ---------------------------------------------------------------- }
constructor TUpdateThread.Create (const AUrl,AVerName,Version : string; const AProxyPar : THttpProxyParams;
                                  const ASSLParams : THttpSSLParams; ALevel : integer;
                                  AOnTermCheck : TStatusEvent; AOnTerminate :TNotifyEvent;
                                  AShowFault : boolean);
begin
  inherited Create (false);
  Priority:=tpLower;
//  FHandle:=CfHandle;
  FUrl:=AUrl; FVerName:=AVerName;
  FVersion:=Version;
  FLevel:=ALevel;
  try
    if AnsiStartsText('https:',AUrl) then with ASSLParams do
      IdHTTP:=TExtHttp.Create(nil,Path,Root,Cert,Key,CertPwd)
    else IdHTTP:=TExtHttp.Create(nil);
    with IdHTTP do begin
      ReadTimeout:=10000;
      HandleRedirects:=true;
      with AProxyPar do begin
        ProxyParams.BasicAuthentication:=UseProxy and (length(UserName)>0) and (length(Password)>0);
        if UseProxy then ProxyParams.ProxyServer:=Server
        else ProxyParams.ProxyServer:='';
        ProxyParams.ProxyPort:=Port;
        ProxyParams.ProxyUsername:=Username;
        ProxyParams.ProxyPassword:=Password;
        end;
      end;
  except
    On E:EIdException do FErrorMsg:=E.Message
    else FErrorMsg:=RSHTTPSSPIUnknwonError;
    end;
  FreeOnTerminate:=true;
  FCanTerminate:=true;
  FCancelled:=false;
  FShowFault:=AShowFault;
  FUpdateStatus:=usNoServer;
  FErrorMsg:='';
  OnCheckTerminated:=AOnTermCheck;
  OnTerminate:=AOnTerminate;
  end;

destructor TUpdateThread.Destroy;
begin
  IdHTTP.Free;
  inherited;
  end;

procedure TUpdateThread.Cancel;
begin
  FCancelled:=true;
  end;

procedure TUpdateThread.SetTermCheck (ACallBack : TStatusEvent);
begin
  FOnTermCheck:=ACallBack;
  end;

procedure TUpdateThread.DoTermCheck;
begin
  if assigned(FOnTermCheck) then FOnTermCheck(FCanTerminate) //Synchronize(TermCheck)
  else FCanTerminate:=true;
  end;

procedure TUpdateThread.Execute;
var
  su,s,v           : string;
begin
  FUpdateStatus:=usNoServer;
  s:=''; su:=FUrl+FVerName;
  if length(su)>0 then begin
    try
      s:=IdHTTP.Get(su);
    except
      On E:EIdException do begin
        FErrorMsg:=E.Message; s:='';
        end
      else begin
        FErrorMsg:=RSHTTPSSPIUnknwonError; s:='';
        end;
      end;
    if length(s)>0 then begin
      s:=CrLfToChar(s,'|');
      v:=ReadNxtStr(s,'|');
      if IsNewerVersion(FVersion,v,FLevel) then begin
        FUpdateStatus:=usAvail;
        FVersion:=v;
        end
      else FUpdateStatus:=usNotAvail;
      end;
    end
  else FErrorMsg:=RSFTPInvalidIP;
//  if FHandle<>0 then PostMessage (FHandle,wmUpdateAvailable,ord(UpdateAvail),0);
  if FCancelled then FUpdateStatus:=usCancel;
//  FUpdateStatus:=usAvail; // zum Testen
  repeat
    Sleep(200);
    DoTermCheck;
    until FCanTerminate;
  Terminate;
  end;

function TUpdateThread.GetDownloadInfo (var AUrl,AName : string; const AltUrl : string ='') : boolean;
var
  s,su : string;
begin
  AName:=''; su:=FUrl;
  if length(FUrl)>0 then begin
    if length(AltUrl)=0 then s:=FUrl+NewExt(FVerName,LocExt)
    else s:=NewExt(AltUrl,LocExt);
    try
      s:=IdHTTP.Get(s);
    except
      On E:EIdException do begin
        FErrorMsg:=E.Message; s:='';
        end
      else begin
        FErrorMsg:=RSHTTPSSPIUnknwonError; s:='';
        end;
      end;
    if length(s)>0 then begin
      s:=CrLfToChar(s,'|');
      AName:=ReadNxtStr(s,'|');
      if length(s)>0 then begin
        su:=AName;  // server path
        AName:=ReadNxtStr(s,'|'); // compatible to old version
        end
      end
    end
  else FErrorMsg:=RSFTPInvalidIP;
  Result:=length(AName)>0;
  AUrl:=su+AName;
  end;

{ ---------------------------------------------------------------- }
procedure TDownload.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

procedure TDownload.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
{$IFDEF ACCESSIBLE}
  if (Key=VK_F11) then begin
    if ActiveControl is TLabeledEdit then with ActiveControl as TLabeledEdit do begin
      if length(Hint)=0 then ShowHintInfo(EditLabel.Caption) else ShowHintInfo(Hint);
      end
    else with ActiveControl do if length(Hint)>0 then ShowHintInfo(Hint);
    end;
{$ENDIF}
  end;

procedure TDownload.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if assigned(IdHTTP) then IdHTTP.Free;
  end;

procedure TDownload.StartWork(Sender: TObject; AWorkMode: TWorkMode; AWorkCount: Int64);
begin
  if AWorkMode=wmRead then nMax:=AWorkCount;
  pbDownload.Position:=0;
  Application.ProcessMessages;
  end;

procedure TDownload.ShowWork(Sender: TObject; AWorkMode: TWorkMode; AWorkCount: Int64);
begin
  if (AWorkMode=wmRead) and (nMax>0) then
    pbDownload.Position:=round((AWorkCount/nMax)*100);
  Application.ProcessMessages;
  end;

procedure TDownload.FinishWork(Sender: TObject; AWorkMode: TWorkMode);
begin
  if AWorkMode=wmRead then pbDownload.Position:=100;
  Application.ProcessMessages;
  end;


{------------------------------------------------------------------- }
function TDownload.Execute (const ATitle,AURL,AFilename : string;
                            const AProxyPar : THttpProxyParams;
                            const ASSLParams : THttpSSLParams;
                            CheckHash : integer; var ErrorMsg : string) : boolean;
var
  fs   : TFileStream;
  sh,smd,sn  : string;
begin
  Caption:=ATitle;
  edURL.Text:=AURL;
  edFilename.Text:=AFilename;
  ErrorMsg:='';
  if AnsiStartsText('https:',AUrl) then with ASSLParams do
    IdHTTP:=TExtHttp.Create(nil,Path,Root,Cert,Key,CertPwd)
  else IdHTTP:=TExtHttp.Create(Application);
  with IdHTTP do begin
    with AProxyPar do begin
      ProxyParams.BasicAuthentication:=UseProxy and (length(UserName)>0) and (length(Password)>0);
      if UseProxy then ProxyParams.ProxyServer:=Server
      else ProxyParams.ProxyServer:='';
      ProxyParams.ProxyPort:=Port;
      ProxyParams.ProxyUsername:=Username;
      ProxyParams.ProxyPassword:=Password;
      end;
    OnWorkBegin:=StartWork;
    OnWork:=ShowWork;
    OnWorkEnd:=FinishWork;
    end;
  Result:=false;
  Show;
  BringToFront;
  try
    fs:=TFileStream.Create(AFilename,fmCreate);
    try
      IdHTTP.Get(AUrl,fs);
      Result:=true;
    except
      on E:EIdException do ErrorMsg:=E.Message;
      end;
    fs.Position:=0;
    if Result and (CheckHash>0) then begin
      if CheckHash=1 then begin
        sh:=NewExt(AUrl,Md5Ext); sn:='MD5';
        end
      else Begin
        sh:=NewExt(AUrl,ShaExt); sn:='SHA256';
        end;
      try smd:=Trim(IdHTTP.Get(sh)); except smd:=''; end;
      Result:=length(smd)>0;
      if Result then begin
        if CheckHash=1 then Result:=AnsiSameText(smd,HashStream(htMD5,fs))
        else Result:=AnsiSameText(smd,HashStream(htSHA256,fs));
        if not Result then ErrorMsg:=TryFormat(dgettext('dialogs','%s verification failed!'),[sn]);
        end
      else ErrorMsg:=TryFormat(dgettext('dialogs','%s hash file not found!'),[sn]);
      end;
  finally
    fs.Free;
    end;
  Close;
  end;

{------------------------------------------------------------------- }
function PerformDownload (const ATitle,AURL,AFilename : string;
                          const AProxyPar : THttpProxyParams;
                          const ASSLParams : THttpSSLParams;
                          CheckHash : integer; var ErrorMsg : string) : boolean;
begin
  if not assigned(Download) then Download:=TDownload.Create(Application);
  Result:=Download.Execute(ATitle,AURL,AFilename,AProxyPar,ASSLParams,CheckHash,ErrorMsg);
  FreeAndNil(Download);
  end;

end.
