(* Delphi-Unit
   Setting of Mail connection parameters
   =====================================

    Dr. J. Rathlev, D-24222 Schwentinental (kontakt(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - May 2007
   last modified: Nov. 2021
   *)

unit MailDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls,
  NumberEd, IdAssignedNumbers;

const
  defSmtpPort = IdPORT_SMTP;

type
  TMailMode = (mmNone,mmAlways,mmError);

  TMailData = record
    SMTP,
    Username,
    Password,
    Sender,
    ReplyTo,
    Recipients,
    Subject       : string;
    Port,
    SecureMode    : integer;
    WriteLog,
    Enabled       : boolean;
    end;

  TMailDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    lbRecipients: TListBox;
    Label5: TLabel;
    Label6: TLabel;
    edSmtpPwd: TEdit;
    Label7: TLabel;
    edPort: TRangeEdit;
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    btnAdd: TBitBtn;
    btnRemove: TBitBtn;
    btnEdit: TBitBtn;
    edSubject: TLabeledEdit;
    gbSecure: TGroupBox;
    rbNone: TRadioButton;
    rbAvail: TRadioButton;
    rbTLS: TRadioButton;
    rbSSL: TRadioButton;
    cbMail: TCheckBox;
    cbMailLog: TCheckBox;
    cbServer: TComboBox;
    cbUser: TComboBox;
    cbSender: TComboBox;
    cbReply: TComboBox;
    bbPassword: TBitBtn;
    cbShowPwd: TCheckBox;
    procedure btnAddClick(Sender: TObject);
    procedure btnRemoveClick(Sender: TObject);
    procedure btnEditClick(Sender: TObject);
    procedure cbShowPwdClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure rbNoneClick(Sender: TObject);
    procedure rbAvailClick(Sender: TObject);
    procedure rbTLSClick(Sender: TObject);
    procedure rbSSLClick(Sender: TObject);
    procedure bbPasswordClick(Sender: TObject);
  private
    { Private declarations }
    FIniName : string;
    MailPwd  : AnsiString;
    hl       : TStringList;
    fSecureMode : integer;
  public
    { Public declarations }
{$IFDEF HDPI}   // scale glyphs and images for High DPI
    procedure AfterConstruction; override;
{$EndIf}
    function DefaultValues : TMailData;
    procedure LoadFromIni (const AIniName : string);
    function Execute (const Prompt : string; ShowPwd : boolean;
                      var AMailDat : TMailData) : boolean;
  end;

var
  MailDialog: TMailDialog;

implementation

{$R *.DFM}

uses GgtDummy, Vcl.Dialogs, WinUtils, System.IniFiles,
  InpText, StringUtils, ExtSysUtils, mMailer10, NewPwdDlg;

const
  SmptSekt = 'SmtpServer';
  UserSekt = 'MailUser';
  MailSekt = 'MailTo';

  iniMName = 'Address';

procedure TMailDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self);
  FIniName:='';
  hl:=TStringList.Create;
  end;

{$IFDEF HDPI}   // scale glyphs and images for High DPI
procedure TMailDialog.AfterConstruction;
begin
  inherited;
  if Application.Tag=0 then
    ScaleButtonGlyphs(self,PixelsPerInchOnDesign,Monitor.PixelsPerInch);
  end;
{$EndIf}

procedure TMailDialog.FormDestroy(Sender: TObject);
begin
  if length(FIniName)>0 then begin
    SaveHistory(FIniName,SmptSekt,true,cbServer);
    SaveHistory(FIniName,UserSekt,true,cbUser);
    SaveHistory(FIniName,MailSekt,iniMName,true,hl);
    end;
  hl.Free;
  end;

procedure TMailDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if ModalResult=mrOK then begin
    if length(cbServer.Text)=0 then begin
      CanClose:=false;
      ErrorDialog(dgettext('dialogs','Geben Sie bitte einen SMTP-Server an!'));
      cbServer.SetFocus;
      end
    else if lbRecipients.Items.Count=0 then begin
      CanClose:=false;
      ErrorDialog(dgettext('dialogs','Geben Sie bitte mindestens einen Empfnger an!'));
      btnAdd.Click;
      end;
    end;
  end;

procedure TMailDialog.LoadFromIni (const AIniName : string);
begin
  // nur beim Neustart laden
  if (length(FIniName)=0) and (length(AIniName)>0) then begin
    FIniName:=AIniName;
    LoadHistory(FIniName,SmptSekt,cbServer);
    LoadHistory(FIniName,UserSekt,cbUser);
    LoadHistory(FIniName,MailSekt,iniMName,hl);
    end;
  end;

procedure TMailDialog.bbPasswordClick(Sender: TObject);
var
  s : AnsiString;
begin
  s:='';
  if GetPassword(TopLeftPos(bbPassword),Caption,s) then begin
    MailPwd:=s; edSmtpPwd.Text:=FillStr(#$25CF,Length(MailPwd));
    end;
  end;

procedure TMailDialog.btnAddClick(Sender: TObject);
var
  t : string;
begin
  t:='';
  if InputText(CursorPos,dgettext('dialogs','Neuer Mail-Empfnger'),
    dgettext('dialogs','Adresse:'),false,'',hl,false,t) then
    with lbRecipients do if Items.IndexOf(t)<0 then ItemIndex:=Items.Add(t);
  end;

procedure TMailDialog.btnRemoveClick(Sender: TObject);
var
  n : integer;
begin
  with lbRecipients do if (Items.Count>0) and (ItemIndex>=0) then begin
    n:=ItemIndex;
    if (ItemIndex>=0) and ConfirmDialog(Caption,
        TryFormat(dgettext('dialogs','Empfnger:'+sLineBreak+'%s'+sLineBreak+'aus der Liste entfernen?'),
        [Items[ItemIndex]])) then begin
      Items.Delete(ItemIndex);
      if n>=Items.Count then n:=Items.Count-1;
      ItemIndex:=n;
      end;
    end;
  end;

procedure TMailDialog.btnEditClick(Sender: TObject);
var
  t : string;
begin
  with lbRecipients do if (Items.Count>0) and (ItemIndex>=0) then begin
    t:=Items[ItemIndex];
    if InputText(CursorPos,dgettext('dialogs','Mail-Empfnger bearbeiten'),
        dgettext('dialogs','Adresse:'),false,'',hl,false,t) then
      Items[ItemIndex]:=t;
    end;
  end;

procedure TMailDialog.cbShowPwdClick(Sender: TObject);
begin
  if cbShowPwd.Checked then edSmtpPwd.Text:=MailPwd
  else edSmtpPwd.Text:=FillStr(#$25CF,Length(MailPwd));
  end;

procedure TMailDialog.rbNoneClick(Sender: TObject);
begin
  fSecureMode:=0;
  edPort.Value:=IdPORT_SMTP;
  end;

procedure TMailDialog.rbAvailClick(Sender: TObject);
begin
  fSecureMode:=1;
  edPort.Value:=IdPORT_SMTP;
  end;

procedure TMailDialog.rbTLSClick(Sender: TObject);
begin
  fSecureMode:=2;
  edPort.Value:=IdPORT_SMTP;
  end;

procedure TMailDialog.rbSSLClick(Sender: TObject);
begin
  fSecureMode:=3;
  edPort.Value:=IdPORT_ssmtp;
  end;

{ ---------------------------------------------------------------- }
function TMailDialog.DefaultValues : TMailData;
begin
  with Result do begin
    SMTP:=''; Port:=defSmtpPort; SecureMode:=0;
    Sender:=''; ReplyTo:=''; Recipients:='';
    Username:=''; Password:=''; Subject:='';
    WriteLog:=false; Enabled:=false;
    end;
  end;

function TMailDialog.Execute (const Prompt : string; ShowPwd : boolean;
                              var AMailDat : TMailData) : boolean;
begin
  cbShowPwd.Visible:=ShowPwd;
  with cbMail do begin
    Caption:=Prompt;
    Visible:=length(Prompt)>0;
    end;
  with AMailDat do begin
    cbServer.Text:=SMTP;
    cbUser.Text:=Username;
    MailPwd:=Password;
    edSmtpPwd.Text:=FillStr(#$25CF,Length(MailPwd));
    with cbSender do begin
      Items.Assign(hl);
      Text:=Sender;
      end;
    with cbReply do begin
      Items.Assign(hl);
      Text:=ReplyTo;
      end;
    lbRecipients.Items.CommaText:=Recipients;
    fSecureMode:=SecureMode;
    case SecureMode of
    1 : rbAvail.Checked:=true;
    2 : rbTLS.Checked:=true;
    3 : rbSSL.Checked:=true;
    else rbNone.Checked:=true;
      end;
    edPort.Value:=Port;
    edSubject.Text:=Subject;
    cbMailLog.Checked:=WriteLog;
    cbMail.Checked:=Enabled;
    end;
  if ShowModal=mrOK then with AMailDat do begin
    SMTP:=cbServer.Text;
    Port:=edPort.Value;
    Username:=cbUser.Text;
    Password:=MailPwd;
    Sender:=cbSender.Text;
    AddToHistory(hl,Sender);
    ReplyTo:=cbReply.Text;
    AddToHistory(hl,ReplyTo);
    Recipients:=lbRecipients.Items.CommaText;
    SecureMode:=fSecureMode;
    Subject:=edSubject.Text;
    WriteLog:=cbMailLog.Checked;
    Enabled:=cbMail.Checked;
    Result:=true;
    end
  else Result:=false;
  end;

end.
